# pylint: disable-msg=W0201
from .utils.test_generator import TestGenerator
from slash.runner import run_tests
from slash.session import Session
from slash.result import Result
import slash
from .utils import TestCase

class InterruptionTest(TestCase):
    def setUp(self):
        super(InterruptionTest, self).setUp()
        self.generator = TestGenerator()
        self.total_num_tests = 5
        self.runnables = [t.generate_test() for t in self.generator.generate_tests(self.total_num_tests)]
        self.interrupted_index = 3
        self.interrupted = self.runnables[self.interrupted_index]
        self.generator.add_test_run_callback(self.interrupted, self._do_test_callback)
        slash.hooks.test_interrupt.register(self._test_interrupt_hook, id(self))
        self.addCleanup(slash.hooks.test_interrupt.unregister_by_identifier, id(self))

        with Session() as session:
            self.session = session
            with self.assertRaises(KeyboardInterrupt):
                run_tests(self.runnables)

    def _do_test_callback(self, _):
        slash.add_cleanup(self._regular_cleanup)
        slash.add_critical_cleanup(self._critical_cleanup)
        raise KeyboardInterrupt()

    _regular_cleanup_called = False
    def _regular_cleanup(self):
        self._regular_cleanup_called = True

    def _critical_cleanup(self):
        self._critical_cleanup_called = True

    def _test_interrupt_hook(self):
        self._hook_called_with_id = slash.context.test.TESTGENERATOR_TEST_ID

    def test_interrupted_run(self):
        for index, test in enumerate(self.runnables):
            if index > self.interrupted_index:
                with self.assertRaises(LookupError):
                    self.session.get_result(test)
                continue
            result = self.session.get_result(test)
            self.assertTrue(result.is_finished())
            if index < self.interrupted_index:
                self.assertTrue(result.is_success())
            else:
                self.assertFalse(result.is_success())

    def test_test_interrupt_hook_called(self):
        self.assertEquals(self._hook_called_with_id, self.interrupted.TESTGENERATOR_TEST_ID)

    def test_regular_cleanups_not_called(self):
        self.assertFalse(self._regular_cleanup_called)

    def test_critical_cleanups_called(self):
        self.assertTrue(self._critical_cleanup_called)



### make nosetests ignore stuff we don't want to run
run_tests.__test__ = False



import itertools

class IDSpace(object):
    def __init__(self, base):
        super(IDSpace, self).__init__()
        if not base.endswith(":"):
            base += ":"
        self._allocator = (base + str(i) for i in itertools.count(1))
    def allocate(self):
        return next(self._allocator)



import functools
from ..exceptions import SkipTest
from ..runnable_test_factory import RunnableTestFactory

def skip_test(*args):
    """
    Skips the current test execution by raising a :class:`slash.exceptions.SkipTest`
    exception. It can optionally receive a reason argument.
    """
    raise SkipTest(*args)

def skipped(thing, reason=None):
    """
    A decorator for skipping methods and classes
    """
    if isinstance(thing, str):
        return functools.partial(skipped, reason=thing)
    if isinstance(thing, type) and issubclass(thing, RunnableTestFactory):
        thing.skip_all(reason)
        return thing

    @functools.wraps(thing)
    def new_func(*_, **__):
        skip_test(reason)
    return new_func



import functools
try:
    from collections import OrderedDict # pylint: disable=E0611
except ImportError: # python 2.6
    from ordereddict import OrderedDict # pylint: disable=F0401

from six import itervalues
from . import ctx
from . import hooks
from . import log
from .exception_handling import handling_exceptions
from .result import Result
from .interfaces import Activatable
from .result import SessionResult
from .utils.id_space import IDSpace
from contextlib import contextmanager
import uuid

class Session(Activatable):
    def __init__(self):
        super(Session, self).__init__()
        self.id = "{0}:0".format(uuid.uuid1())
        self.id_space = IDSpace(self.id)
        self._complete = False
        self._context = None
        self._results = OrderedDict()
        #: an aggregate result summing all test results and the global result
        self.result = SessionResult(functools.partial(itervalues, self._results))
    def create_result(self, test):
        assert test.__slash__.id not in self._results
        returned = Result(test.__slash__)
        self._results[test.__slash__.id] = returned
        return returned
    def get_result(self, test):
        if test.__slash__ is None:
            raise LookupError("Could not find result for {0}".format(test))
        return self._results[test.__slash__.id]
    def activate(self):
        assert self._context is None
        self._context = _session_context(self)
        with handling_exceptions():
            self._context.__enter__()

    def deactivate(self):
        self.result.global_result.mark_finished()
        with handling_exceptions():
            self._context.__exit__(None, None, None)
    def mark_complete(self):
        self._complete = True
    def is_complete(self):
        return self._complete


@contextmanager
def _session_context(session):
    assert ctx.context.session is None
    ctx.push_context()
    ctx.context.session = session
    try:
        with log.get_session_logging_context():
            hooks.session_start()
            try:
                yield
            finally:
                hooks.session_end()
    finally:
        ctx.pop_context()




from .utils import TestCase
import slash
import slash.runner
from slash.session import Session
from slash.loader import Loader

class CleanupsTest(TestCase):
    def test_cleanups(self):
        class Test(slash.Test):
            def test1(self_):
                self.events.test1()
                slash.add_cleanup(self.events.cleanup, "test1 cleanup")
            def test2(self_):
                self.events.test2()
                slash.add_cleanup(self.events.cleanup, "test2 cleanup")

        with self.forge.any_order():
            with self.forge.ordered():
                self.events.test1()
                self.events.cleanup("test1 cleanup")
            with self.forge.ordered():
                self.events.test2()
                self.events.cleanup("test2 cleanup")

        self.forge.replay()
        with Session():
            slash.runner.run_tests(Loader().iter_test_factory(Test))
    def test_error_cleanups(self):
        class Test(slash.Test):
            def test(self_):
                slash.add_cleanup(self.events.cleanup, 1)
                slash.add_cleanup(self.events.cleanup, 2)
                raise Exception("!!!")
        self.events.cleanup(1).and_raise(FirstException())
        self.events.cleanup(2).and_raise(SecondException())
        self.forge.replay()
        with Session() as session:
            slash.runner.run_tests(Loader().iter_test_factory(Test))
        [result] = session.result.iter_test_results()
        [err1, err2, err3] = result.get_errors()


class FirstException(Exception):
    pass
class SecondException(Exception):
    pass
class ThirdException(Exception):
    pass



from .metadata import ensure_slash_metadata

class RunnableTestFactory(object):

    @classmethod
    def generate_tests(cls):
        """
        Generates :class:`.RunnableTest` instances to run

        Do not override this method directly. Use :func:`.RunnableTestFactory._generate_tests` instead.
        """
        for test in cls._generate_tests():
            ensure_slash_metadata(test).factory = cls
            yield test

    @classmethod
    def _generate_tests(cls):
        raise NotImplementedError() # pragma: no cover

    __slash_needed_contexts__ = None




# pylint: disable-msg=W0201
from .utils.test_generator import TestGenerator
from .utils import TestCase
from slash.runner import run_tests
from slash.session import Session
from slash.result import Result
from slash.ctx import context
import six # pylint: disable=F0401
import random

class TestRunningTestBase(TestCase):
    def setUp(self):
        super(TestRunningTestBase, self).setUp()
        self.generator = TestGenerator()
        self.total_num_tests = 10
        self.runnables = [t.generate_test() for t in self.generator.generate_tests(self.total_num_tests)]
        self.configure()
        with Session() as session:
            self.session = session
            context.current_test_generator = self.generator
            self.prepare_runnables()
            run_tests(self.runnables)
        self.assertEquals(
            self.session.is_complete(),
            self.should_be_complete(),
            "Session unexpectedly complete" if self.session.is_complete() else "Session unexpectedly incomplete"
        )
    def prepare_runnables(self):
        pass
    def configure(self):
        pass
    def should_be_complete(self):
        return True

class AllSuccessfulTest(TestRunningTestBase):

    def test_all_executed(self):
        self.generator.assert_all_run()

    def test_iter_results_ordering(self):
        results = list(self.session.result.iter_test_results())
        for result, test in zip(results, self.runnables):
            self.assertIs(result.test_metadata, test.__slash__)

_RESULT_PREDICATES = set([
    getattr(Result, method_name)
    for method_name in dir(Result) if method_name.startswith("is_")
    ])

class FailedItemsTest(TestRunningTestBase):
    def prepare_runnables(self):
        num_unsuccessfull = len(self.runnables) // 2
        num_error_tests = 2
        assert 1 < num_unsuccessfull < len(self.runnables)
        unsuccessful = random.sample(self.runnables, num_unsuccessfull)
        self.error_tests = [unsuccessful.pop(-1) for _ in six.moves.xrange(num_error_tests)]
        self.skipped_tests = [unsuccessful.pop(-1)]
        self.failed_tests = unsuccessful
        assert self.error_tests and self.failed_tests
        for failed_test in self.failed_tests:
            self.generator.make_test_fail(failed_test)
        for skipped_test in self.skipped_tests:
            self.generator.make_test_skip(skipped_test)
        for error_test in self.error_tests:
            self.generator.make_test_raise_exception(error_test)

    def test_all_executed(self):
        self.generator.assert_all_run()

    def test_failed_items_failed(self):
        self._test_results(self.failed_tests, [Result.is_finished, Result.is_failure, Result.is_just_failure])

    def test_error_items_error(self):
        self._test_results(self.error_tests, [Result.is_finished, Result.is_error])

    def test_skipped_items_skipped(self):
        self._test_results(self.skipped_tests, [Result.is_finished, Result.is_skip])

    def _test_results(self, tests, true_predicates):
        true_predicates = set(true_predicates)
        assert _RESULT_PREDICATES >= true_predicates, "{0} is not a superset of {1}".format(_RESULT_PREDICATES, true_predicates)
        for test in tests:
            result = self.session.get_result(test)
            for predicate in _RESULT_PREDICATES:
                predicate_result = predicate(result)
                self.assertEquals(predicate_result,
                                  predicate in true_predicates,
                                  "Predicate {0} unexpectedly returned {1}".format(predicate, predicate_result))

class StopOnFailuresTest(TestRunningTestBase):
    def configure(self):
        self.override_config("run.stop_on_error", True)
    def prepare_runnables(self):
        self.num_successful = 4
        self.generator.make_test_raise_exception(self.runnables[self.num_successful])
        # make sure that skips are not considered failures
        self.generator.make_test_skip(self.runnables[1])
    def should_be_complete(self):
        return False
    def test_stopped_after_failure(self):
        for index, runnable in enumerate(self.runnables):
            if index <= self.num_successful:
                result = self.session.get_result(runnable)
                self.assertTrue(result.is_finished())
            else:
                with self.assertRaises(LookupError):
                    self.session.get_result(runnable)

### make nosetests ignore stuff we don't want to run
run_tests.__test__ = False



from .exceptions import CannotLoadTests
from .exception_handling import handling_exceptions
from .runnable_test_factory import RunnableTestFactory
from .utils.imports import import_file
from .ctx import context
from contextlib import contextmanager
from logbook import Logger # pylint: disable=F0401
from six import iteritems # pylint: disable=F0401
import os
import sys

_logger = Logger(__name__)

class Loader(object):
    """
    Provides iteration interfaces to load runnable tests from various places
    """
    def iter_path(self, path):
        return self.iter_paths([path])
    def iter_paths(self, paths):
        paths = list(paths)
        for path in paths:
            if not os.path.exists(path):
                raise CannotLoadTests("Path {0} could not be found".format(path))
        for path in paths:
            for file_path in _walk(path):
                _logger.debug("Checking {0}", file_path)
                if not self._is_file_wanted(file_path):
                    _logger.debug("{0} is not wanted. Skipping...", file_path)
                    continue
                with self._handling_import_errors():
                    module = import_file(file_path)
                for runnable in self._iter_runnable_tests_in_module(module):
                    yield runnable

    @contextmanager
    def _handling_import_errors(self):
        try:
            with handling_exceptions(context="during import"):
                yield
        except Exception: # pylint: disable=W0703
            if not context.session:
                raise
            context.session.result.global_result.add_error()

    def iter_test_factory(self, factory):
        for test in factory.generate_tests():
            yield test

    def iter_package(self, package_name):
        if package_name not in sys.modules:
            __import__(package_name)
        path = sys.modules[package_name].__file__
        if os.path.basename(path) in ("__init__.py", "__init__.pyc"):
            path = os.path.dirname(path)
        return self.iter_path(path)

    def _is_file_wanted(self, filename):
        return filename.endswith(".py")

    def _iter_runnable_tests_in_module(self, module):
        for factory_name, factory in iteritems(vars(module)):
            if factory is RunnableTestFactory: # probably imported directly
                continue
            if isinstance(factory, type) and issubclass(factory, RunnableTestFactory):
                _logger.debug("Getting tests from {0}:{1}..", module, factory_name)
                for test in self.iter_test_factory(factory):
                    yield test

def _walk(p):
    if os.path.isfile(p):
        return [p]
    return (os.path.join(dirname, filename)
            for dirname, _, filenames in os.walk(p)
            for filename in filenames)



from .utils import TestCase
import slash
from slash.session import Session
from slash import context

class GlobalsTest(TestCase):
    def setUp(self):
        super(GlobalsTest, self).setUp()
        self.session = Session()
    def test_get_current_session(self):
        with self.session:
            self.assertIs(context.session, self.session)
    def test_get_current_test(self):
        with self.session:
            self.assertIsNone(context.test)
            self.assertIsNone(context.test_id)
            parent_test = self
            class Test(slash.Test):
                def test(self):
                    parent_test.assertIs(context.test, self)
                    parent_test.assertEquals(context.test_id, self.__slash__.id)
            slash.runner.run_tests(Test.generate_tests())
        self.assertTrue(self.session.result.is_success())



from ..app import get_application_context
from ..exceptions import SlashException
from ..runner import run_tests
from ..utils import cli_utils
import logbook
import sys

_logger = logbook.Logger(__name__)

def slash_run(args, report_stream=sys.stderr):
    with get_application_context(
            argv=args, args=_get_extra_cli_args(),
            enable_interactive=True,
            report_stream=report_stream) as app:
        if not app.args.paths and not app.args.interactive:
            app.error("No tests specified")
        try:
            run_tests(app.test_loader.iter_paths(app.args.paths))
        except SlashException as e:
            logbook.error(e)
            return -1
        if app.session.result.is_success():
            return 0
        return -1

def _get_extra_cli_args():
    return [
        cli_utils.Argument(
            "paths", metavar="TEST", nargs="*",
            help="Test name to run. This can be either a file or a test FQDN. "
            "See documentation for details"),
        ]



from .utils import (
    TestCase,
    CustomException,
    )
from contextlib import contextmanager
from forge import Anything
from slash import exception_handling
from slash.utils import debug
from slash.exceptions import SkipTest

class ExceptionMarksTest(TestCase):
    def test_exception_mark(self):
        e = CustomException()
        mark = object()
        self.assertFalse(exception_handling.is_exception_marked(e, "a"))
        self.assertFalse(exception_handling.is_exception_marked(e, "b"))
        exception_handling.mark_exception(e, "a", mark)
        self.assertTrue(exception_handling.is_exception_marked(e, "a"))
        self.assertFalse(exception_handling.is_exception_marked(e, "b"))
        self.assertIs(exception_handling.get_exception_mark(e, "a"), mark)

class HandlingExceptionsContextTest(TestCase):
    def setUp(self):
        super(HandlingExceptionsContextTest, self).setUp()
        self.handler = self.forge.create_wildcard_mock()
        self.forge.replace_with(exception_handling, "_EXCEPTION_HANDLERS", [self.handler])
        # expect handler to be called once!
        self.raised = CustomException()
        self.handler((CustomException, self.raised, Anything()))
        self.forge.replay()
    def test_handling_exceptions(self):
        with self.assertRaises(CustomException) as caught:
            with exception_handling.handling_exceptions():
                with exception_handling.handling_exceptions():
                    with exception_handling.handling_exceptions():
                        raise self.raised
        self.assertIs(caught.exception, self.raised)

class DebuggingTest(TestCase):
    def setUp(self):
        super(DebuggingTest, self).setUp()
        self.orig_debuggers = debug._KNOWN_DEBUGGERS
        self.addCleanup(setattr, debug, "_KNOWN_DEBUGGERS", self.orig_debuggers)
        self.debugger_called = False
        debug._KNOWN_DEBUGGERS = [self.dummy_debugger]
    def dummy_debugger(self, *args, **kwargs):
        self.debugger_called = True
    def test_debugging_not_configured(self):
        self._raise_exception_in_context(ZeroDivisionError)
        self.assertFalse(self.debugger_called)
    def test_debugging_configured_no_skips(self):
        self.override_config("debug.debug_skips", False)
        self.override_config("debug.enabled", True)
        self._raise_exception_in_context(SkipTest)
        self.assertFalse(self.debugger_called)
    def test_debugging_skips(self):
        self.override_config("debug.debug_skips", True)
        self.override_config("debug.enabled", True)
        self._raise_exception_in_context(SkipTest)
        self.assertTrue(self.debugger_called)
    def _raise_exception_in_context(self, exception_type):
        with self.assertRaises(exception_type):
            with exception_handling.handling_exceptions():
                raise exception_type()

class ExceptionSwallowingTest(TestCase):
    def test_swallow(self):
        with exception_handling.get_exception_swallowing_context():
            raise CustomException("!!!")
    def test_no_swallow(self):
        with self.assertNoSwallow() as raised:
            raise exception_handling.noswallow(raised)
    def test_disable_exception_swallowing_function(self):
        with self.assertNoSwallow() as raised:
            exception_handling.disable_exception_swallowing(raised)
            raise raised
    def test_disable_exception_swallowing_decorator(self):
        @exception_handling.disable_exception_swallowing
        def func():
            raise raised
        with self.assertNoSwallow() as raised:
            func()
    @contextmanager
    def assertNoSwallow(self):
        raised = CustomException()
        with self.assertRaises(CustomException) as caught:
            with exception_handling.get_exception_swallowing_context():
                yield raised
        self.assertIs(raised, caught.exception)



import sys
from . import log
from . import hooks
from .cleanups import call_cleanups
from .conf import config
from .ctx import context
from .exceptions import (
    TestFailed,
    SkipTest,
    )
from .exception_handling import handling_exceptions
from .metadata import ensure_slash_metadata
from .test_context import get_test_context_setup
from .utils.peekable_iterator import PeekableIterator
from contextlib import contextmanager
import logbook # pylint: disable=F0401

_logger = logbook.Logger(__name__)

def run_tests(iterable):
    """
    Runs tests from an iterable using the current session
    """
    test_iterator = PeekableIterator(iterable)
    for test in test_iterator:
        ensure_slash_metadata(test).id = context.session.id_space.allocate()
        _logger.debug("Running {0}...", test)
        with _get_test_context(test):
            with _get_test_hooks_context():
                with _update_result_context() as result:
                    with _cleanup_context():
                        with handling_exceptions():
                            with get_test_context_setup(test, test_iterator.peek_or_none()):
                                test.run()
        if not result.is_success() and not result.is_skip() and config.root.run.stop_on_error:
            _logger.debug("Stopping (run.stop_on_error==True)")
            break
    else:
        context.session.mark_complete()

@contextmanager
def _cleanup_context():
    # In Python 2.6, exc_info() in finally is sometimes (None, None, None), so we capture it explicitly
    exc_info = None
    try:
        try:
            yield
        except:
            exc_info = sys.exc_info()
            raise
    finally:
        call_cleanups(critical_only=exc_info is not None and exc_info[0] is KeyboardInterrupt)
        del exc_info

@contextmanager
def _get_test_context(test):
    with _set_current_test_context(test):
        with log.get_test_logging_context():
            yield

@contextmanager
def _get_test_hooks_context():
    hooks.test_start()
    try:
        yield
    except SkipTest:
        hooks.test_skip()
    except TestFailed:
        hooks.test_failure()
    except KeyboardInterrupt:
        hooks.test_interrupt()
        raise
    except:
        hooks.test_error()
    else:
        hooks.test_success()
    finally:
        hooks.test_end()

@contextmanager
def _set_current_test_context(test):
    prev_test = context.test
    prev_test_id = context.test_id
    context.test = test
    context.test_id = test.__slash__.id
    try:
        yield
    finally:
        context.test = prev_test
        context.test_id = prev_test_id

@contextmanager
def _update_result_context():
    result = context.session.create_result(context.test)
    try:
        try:
            yield result
        except:
            _logger.debug("Exception escaped test", exc_info=sys.exc_info())
            raise
    except SkipTest as e:
        result.add_skip(e.reason)
        raise
    except TestFailed:
        result.add_failure()
        raise
    except KeyboardInterrupt:
        result.mark_interrupted()
        raise
    except:
        result.add_error()
        raise
    finally:
        result.mark_finished()



import itertools
import slash
from .utils import TestCase
from .utils import run_tests_assert_success
from .utils.event_recorder import EventRecorder

_recorder = None

class _SampleContextBase(slash.TestContext):
    def before(self):
        self._record("before")
    def after(self):
        self._record("after")
    def before_case(self):
        self._record("before_case")
    def after_case(self):
        self._record("after_case")
    def _record(self, event_name):
        event_name = "{0}.{1}".format(type(self).__name__, event_name)
        if "_case" in event_name:
            test = slash.context.test
            event_name += "({0}.{1})".format(
                type(test).__name__,
                test._test_method_name,
            )
        _recorder.record(event_name)

class Context1(_SampleContextBase):
    pass

class Context2(_SampleContextBase):
    pass

class EventRecordingTest(TestCase):
    def setUp(self):
        super(EventRecordingTest, self).setUp()
        global _recorder
        self._events = EventRecorder()
        _recorder = self._events
    def tearDown(self):
        global _recorder
        _recorder = None
        super(EventRecordingTest, self).tearDown()

class MultipleContextsTest(EventRecordingTest):
    def test_multiple_contexts(self):
        @slash.with_context(Context1)
        @slash.with_context(Context2)
        class Test1(slash.Test):
            def test_1(self):
                assert _recorder["Context1.before_case(Test1.test_1)"].timestamp < \
                       _recorder["Context2.before_case(Test1.test_1)"].timestamp, "Context1 did not happen before Context2"

        @slash.with_context(Context2)
        class Test2(slash.Test):
            def test_1(self):
                assert "Context1.before_case(Test2.test_1)" not in _recorder.events, "Context1 unexpectedly called"
                assert _recorder.events["Context2.before_case(Test2.test_1)"].happened

        run_tests_assert_success(itertools.chain(Test1.generate_tests(), Test2.generate_tests()))



import traceback

class Error(object):
    def __init__(self, exc_info):
        super(Error, self).__init__()
        self.exception_type, self.exception, tb = exc_info
        self.exception_text = "".join(traceback.format_exception(
            self.exception_type, self.exception, tb
        ))
    def __repr__(self):
        return repr(self.exception)
    def __str__(self):
        return str(self.exception)



from .utils.callback import Callback
import six

session_start = Callback(doc="Called right after session starts")
session_end   = Callback(doc="Called right before the session ends, regardless of the reason for termination")

test_interrupt = Callback(doc="Called when a test is interrupted by a KeyboardInterrupt or other similar means")
test_start   = Callback(doc="Called right after a test starts")
test_end     = Callback(doc="Called right before a test ends, regardless of the reason for termination")
test_success = Callback(doc="Called on test success")
test_error   = Callback(doc="Called on test error")
test_failure = Callback(doc="Called on test failure")
test_skip    = Callback(doc="Called on test skip")

result_summary = Callback(doc="Called at the end of the execution, when printing results")

exception_caught_before_debugger = Callback(
    doc="Called whenever an exception is caught, but a debugger hasn't been entered yet"
)
exception_caught_after_debugger = Callback(
    doc="Called whenever an exception is caught, and a debugger has already been run"
)

def get_all_hooks():
    for name, callback in six.iteritems(globals()):
        if not isinstance(callback, Callback):
            continue
        yield name, callback



from .utils import TestCase
from .utils import no_op
from .utils import NullFile
from .utils.test_generator import TestGenerator
from slash.frontend import slash_run
from slash import site
import os
import shutil

class MissingFilesTest(TestCase):
    def test_slash_run_fails_fast_for_missing_files(self):
        result = slash_run.slash_run(["/non/existing/path"], report_stream=NullFile())
        self.assertNotEquals(result, 0, "slash run unexpectedly succeeded for a missing path")

class SlashRunTest(TestCase):
    def setUp(self):
        super(SlashRunTest, self).setUp()
        self.generator = TestGenerator()
        make_test = self.generator.generate_test
        self.forge.replace_with(site, "load", no_op)
        self.root_path = self.generator.write_test_directory(
            {
                "test_1.py" : make_test(),
                "dir_1" : {
                    "dir_2" : {
                        "test_2.py" : [make_test(), make_test()],
                    },
                    "test_3.py" : make_test(),
                    "regular_file.txt" : "some content here",
                    "other_regular_file" : "more contents",
                }
            }
        )
        self.addCleanup(shutil.rmtree, self.root_path)
    def test_slash_run_directory_success(self):
        result = self._execute_slash_run([self.root_path])
        self.assertEquals(result, 0, "slash run did not return 0 on success")
    def test_slash_run_directory_failure(self):
        self._test__slash_run_directory_unsuccessful(self.generator.make_test_fail)
    def test_slash_run_directory_error(self):
        self._test__slash_run_directory_unsuccessful(self.generator.make_test_raise_exception)
    def _test__slash_run_directory_unsuccessful(self, fault):
        expected = self.generator.get_expected_test_ids()
        fault(expected[2])
        result = self._execute_slash_run([self.root_path])
        self.assertNotEquals(result, 0, "slash run unexpectedly returned 0 for failure")
    def test_slash_run_specific_file(self):
        for path in [
                "test_1.py",
                "dir_1/dir_2/test_2.py",
                "dir_1/test_3.py"
        ]:
            result = self._execute_slash_run([os.path.join(self.root_path, path)])
            self.assertEquals(result, 0, "slash run did not return successfully for {0}".format(path))
    def _execute_slash_run(self, argv):
        return slash_run.slash_run(argv, report_stream=NullFile())
    def tearDown(self):
        self.generator.assert_all_run()
        super(SlashRunTest, self).tearDown()



from .utils import TestCase
import slash

class SkipTestTest(TestCase):
    def test_skip_test(self):
        "Make sure the skip_test function raises a SkipTest exception"
        for args in [
                (), ("message",)
        ]:
            with self.assertRaises(slash.exceptions.SkipTest) as caught:
                slash.skip_test(*args)
            if args:
                self.assertEquals(caught.exception.reason, args[0])

class SkipWithBeforeAfterTest(TestCase):
    def test(self):
        "Make sure that after() is called for Test even if we skip"
        parent_test = self
        class MyTest(slash.Test):
            def test(self):
                slash.skip_test("!")
            def after(self):
                parent_test.after_called = True
        [test] = MyTest.generate_tests()
        with self.assertRaises(slash.exceptions.SkipTest):
            test.run()
        self.assertTrue(self.after_called, "after() was not called upon skip")

class SkipDecoratorTest(TestCase):
    def test_method_without_reason(self):
        class Test(slash.Test):
            @slash.skipped
            def test(self):
                pass
        [test] = Test.generate_tests()
        self.assert_skips(test.run)
    def test_method_with_reason(self):
        class Test(slash.Test):
            @slash.skipped("reason")
            def test(self):
                pass
        [test] = Test.generate_tests()
        self.assert_skips(test.run, "reason")
    def test_class_decorator(self):
        @slash.skipped("reason")
        class Test(slash.Test):
            def test_1(self):
                pass
            def test_2(self):
                pass

        [test_1, test_2] = Test.generate_tests()

        self.assert_skips(test_1.run, "reason")
        self.assert_skips(test_2.run, "reason")
    def assert_skips(self, thing, reason=None):
        with self.assertRaises(slash.exceptions.SkipTest) as caught:
            thing()
        self.assertEquals(caught.exception.reason, reason)



import logbook
from six.moves import cStringIO
from .utils import TestCase
from .utils.test_generator import TestGenerator
from slash.frontend import slash_run

class SlashRunTest(TestCase):
    def setUp(self):
        super(SlashRunTest, self).setUp()
        self.generator = TestGenerator()
        (self.success_test, self.fail_test,
         self.error_test, self.skip_test) = self.generator.generate_tests(4)
        self.generator.make_test_skip(self.skip_test)
        self.generator.make_test_fail(self.fail_test)
        self.generator.make_test_raise_exception(self.error_test)

        self.root_path = self.generator.write_test_directory(
            {
                "success_test.py" : self.success_test,
                "fail_test.py" : self.fail_test,
                "error_test.py" : self.error_test,
                "skip_test.py" : self.skip_test
            }
        )
        self.report_stream = cStringIO()
        self.separator = "-" * 80
    def _get_output_part(self, part):
        self.report_stream.seek(0)
        return self.report_stream.getvalue().split(self.separator)[part]
    def get_live_part(self):
        return self._get_output_part(0)
    def get_exceptions_part(self):
        return self._get_output_part(1)
    def get_summary_part(self):
        return self._get_output_part(2)
    def _slash_run(self):
        return slash_run.slash_run([self.root_path], report_stream=self.report_stream)
    def test_live_concise(self):
        self.override_config("log.console_level", logbook.WARNING)
        self._slash_run()
        result = self.get_live_part()
        self.assertEqual(set(result) - set("\n"), set("EFS."))
        self.assertEqual(len(result), 5)
    def test_live_verbose(self):
        self.override_config("log.console_level", logbook.INFO)
        self._slash_run()
        output = self.get_live_part()
        self.assertIn('error_test.{0} ... error\n Traceback'.format(self.error_test._test_class_name), output)
        self.assertIn('fail_test.{0} ... fail\n Traceback'.format(self.fail_test._test_class_name), output)
        self.assertIn('skip_test.{0} ... skip\n Reason here'.format(self.skip_test._test_class_name), output)
        self.assertIn('success_test.{0} ... ok'.format(self.success_test._test_class_name), output)
    def test_summary(self):
        self._slash_run()
        output = self.get_summary_part()
        headers, values = output.strip().splitlines()
        self.assertEqual(values.split(), ['1'] * 4)
    def test_exceptions_summary_verbose(self):
        self.override_config("log.console_level", logbook.INFO)
        self._slash_run()
        output = self.get_exceptions_part()
        self.assertEqual(output.count('>'), 3) # skip, error and fail
        self.assertEqual(output.count('Traceback'), 2) # error and fail
        self.assertIn("OSError: Sample exception", output) # error
        self.assertIn("TestFailed: Test failed", output) # fail
    def test_exceptions_summary_concise(self):
        self.override_config("log.console_level", logbook.WARNING)
        self._slash_run()
        output = self.get_exceptions_part()
        self.assertEqual(output.count('>'), 3) # skip, error and fail
        self.assertEqual(output.count('Traceback'), 0) # not tracebacks!
        self.assertIn("Sample exception", output) # error
        self.assertIn("Test failed", output) # fail



class RunnableTest(object):
    """
    This class is meant to serve as a base class to any test that can
    actually be executed by the Slash runner.
    """
    __slash__ = None
    def run(self):
        """
        This method is meant to be overriden by derived classes to actually
        perform the test logic
        """
        raise NotImplementedError() # pragma: no cover

    def get_canonical_name(self):
        return "{0}.{1}".format(type(self).__module__, type(self).__name__)
    def __repr__(self):
        return repr(self.__slash__)



from ..interface import PluginInterface
from ...conf import config
from ...ctx import session
import requests

class Plugin(PluginInterface):
    def get_name(self):
        return "notifications"
    def session_end(self):
        self._notify("Session Ended", "Session {0} ended".format(session.id))
    def _notify(self, title, message):
        for url, api_key in [
                ("https://prowl.weks.net/publicapi/add", config.root.notifications.prowl_api_key),
                ("https://www.notifymyandroid.com/publicapi/notify", config.root.notifications.nma_api_key),
                ]:
            if api_key is None:
                continue
            resp = requests.post(url, {"apikey": api_key, "application": "slash", "event": title, "description": message})
            resp.raise_for_status()



import itertools
import logbook
import sys
import six

from ..conf import config

_declaration_index = itertools.count()
_logger = logbook.Logger(__name__)

class Callback(object):
    """
    Implements a hook to which callbacks can be registered
    """
    def __init__(self, arg_names=(), doc=""):
        super(Callback, self).__init__()
        self._arg_names = arg_names
        self._callbacks = []
        self.declaration_index = next(_declaration_index)
        self.doc = doc
    def get_argument_names(self):
        return set(self._arg_names)
    def __call__(self, **kwargs):
        last_exc_info = None
        for (_, callback) in self._callbacks:
            try:
                callback(**kwargs)
            except:
                _logger.warn("Ignoring error occurred while calling {0}", callback, exc_info=sys.exc_info())
                if last_exc_info is None:
                    last_exc_info = sys.exc_info()
        if last_exc_info and not config.root.hooks.swallow_exceptions:
            six.reraise(*last_exc_info) # pylint: disable=W0142
    def register(self, func, identifier=None):
        """
        Registers a function to this callback.

        Optional argument identifier for later removal by :func:`slash.utils.callback.Callback.unregister_by_identifier`.
        """
        self._callbacks.append((identifier, func))
        return func # useful for decorators

    def unregister_by_identifier(self, identifier):
        """
        Unregisters a callback identified by ``identifier``.
        """
        for index, (callback_id, _) in reversed(list(enumerate(self._callbacks))):
            if callback_id == identifier:
                self._callbacks.pop(index)

    def iter_registered(self):
        """
        Yields tuples of (identifier, callback) for each registered callback
        """
        return iter(self._callbacks)



from .utils import TestCase
import os
import sys
from tempfile import mktemp
import slash
import slash.site
from slash.frontend import slash_run
import requests
import pkg_resources
from six.moves import cStringIO as StringIO

site_customized = False

class SlashRunSiteCustomizationTest(TestCase):
    "Make sure ``slash run`` calls site.load()"
    def setUp(self):
        super(SlashRunSiteCustomizationTest, self).setUp()
        self.forge.replace(slash.site, "load")
        self.forge.replace_with(sys, "stderr", StringIO())
    def test_slash_run_calls_site_load(self):
        slash.site.load()
        self.forge.replay()
        with self.assertRaises(SystemExit):
            slash_run.slash_run([])

class CustomizationTest(TestCase):
    def get_customization_source(self):
        return "import {0}; {0}.site_customized=True".format(__name__)
    def get_customization_function(self):
        def _customize():
            global site_customized
            site_customized = True
        return _customize
    def test_customize_via_slashrc(self):
        slashrc_path = mktemp()
        self.forge.replace(os.path, "expanduser")
        os.path.expanduser("~/.slash/slashrc").whenever().and_return(slashrc_path)
        with open(slashrc_path, "w") as f:
            f.write(self.get_customization_source())
        self.forge.replay()
        self.assert_customization_loaded()
    def test_customize_via_env_var(self):
        os.environ["SLASH_SETTINGS"] = custom_filename = mktemp()
        self.addCleanup(os.environ.pop, "SLASH_SETTINGS")
        with open(custom_filename, "w") as f:
            f.write(self.get_customization_source())
        self.assert_customization_loaded()
    def test_customize_via_url(self):
        url = "http://nonexistent.com/some/path/to/custom/file.py"
        self.forge.replace(requests, "get")
        fake_response = self.forge.create_mock(requests.Response)
        fake_response.raise_for_status().whenever()
        fake_response.content = self.get_customization_source()
        requests.get(url).and_return(fake_response)
        os.environ["SLASH_SETTINGS"] = url
        self.addCleanup(os.environ.pop, "SLASH_SETTINGS")
        self.forge.replay()
        self.assert_customization_loaded()
    def test_customize_via_pkgutil_entry_point(self):
        self.forge.replace(pkg_resources, "iter_entry_points")
        entry_point = self.forge.create_wildcard_mock()
        pkg_resources.iter_entry_points("slash.site.customize").and_return(iter([entry_point]))
        entry_point.load().and_return(self.get_customization_function())
        self.forge.replay()
        self.assert_customization_loaded()
    def assert_customization_loaded(self):
        global site_customized
        site_customized = False
        slash.site.load()
        self.assertTrue(site_customized, "Customization not loaded!")



import colorama
from six import string_types
import logbook
import itertools
from contextlib import contextmanager
from .formatter import Formatter
from .hooks_context_manager import HooksContextManager
from ..conf import config
from ..ctx import context

def _is_concise():
    return config.root.log.console_level > logbook.NOTICE


@contextmanager
def report_context(report_stream):
    live_reporter_type = ConciseLiveReporter if _is_concise() else VerboseLiveReporter
    with live_reporter_type(report_stream):
        yield
    if _is_concise():
        report_stream.write("\n")
    SummaryReporter(report_stream).report_session(context.session)

def _format_unsuccessfull(formatter, result):
    with formatter.indented():
        for x in itertools.chain(result.get_failures(), result.get_errors(), result.get_skips()):
            if isinstance(x, string_types) or _is_concise():
                formatter.writeln(x)
            else:
                formatter.writeln(x.exception_text)

def _build_colorizer(fore_color_name):
    fore_color = getattr(colorama.Fore, fore_color_name.upper())
    return "{color}{{0}}{reset}".format(color=fore_color, reset=colorama.Fore.RESET).format  # pylint: disable=E1101

_REPORT_COLUMNS = [
    ("Successful", "get_num_successful", _build_colorizer("green")),
    ("Failures", "get_num_failures", _build_colorizer("red")),
    ("Errors", "get_num_errors", _build_colorizer("red")),
    ("Skipped", "get_num_skipped", _build_colorizer("yellow")),
    ]

class SummaryReporter(object):
    def __init__(self, stream):
        super(SummaryReporter, self).__init__()
        self._formatter = Formatter(stream)
        self._colorize = stream.isatty()
    def report_session(self, session):
        self._describe_unsuccessful(session)
        self._describe_summary(session)
    def _describe_unsuccessful(self, session):
        self._formatter.write_separator()
        for result in session.result.iter_all_results():
            if result.is_success():
                continue
            if result.test_metadata is None:
                self._formatter.writeln("> (Outside tests)")
            else:
                self._formatter.writeln("> ", result.test_metadata)
            _format_unsuccessfull(self._formatter, result)
    def _describe_summary(self, session):
        self._formatter.write_separator()

        summary = dict(
            (column_title, getattr(session.result, method_name)())
            for column_title, method_name, _ in _REPORT_COLUMNS
        )

        for is_header in (True, False):
            for column_title, _, colorizer in _REPORT_COLUMNS:
                count = summary[column_title]
                if is_header:
                    s = column_title
                else:
                    s = str(count)
                column_width = len(column_title) + 2
                if count and self._colorize:
                    colorized = colorizer(s)
                    column_width += len(colorized) - len(s)
                    s = colorized
                self._formatter.write(s.ljust(column_width))
            self._formatter.writeln()

class BaseLiveReporter(HooksContextManager):
    def __init__(self, report_stream):
        super(BaseLiveReporter, self).__init__()
        self._formatter = Formatter(report_stream)

class VerboseLiveReporter(BaseLiveReporter):
    def on_test_start(self):
        self._formatter.write("> ", context.test)
        self._formatter.write(" ... ")
    def on_test_success(self):
        self._formatter.writeln("ok")
    def on_test_error(self):
        self._formatter.writeln("error")
        _format_unsuccessfull(self._formatter, context.session.get_result(context.test))
    def on_test_failure(self):
        self._formatter.writeln("fail")
        _format_unsuccessfull(self._formatter, context.session.get_result(context.test))
    def on_test_skip(self):
        self._formatter.writeln("skip")
        _format_unsuccessfull(self._formatter, context.session.get_result(context.test))

class ConciseLiveReporter(BaseLiveReporter):
    def on_test_success(self):
        self._formatter.write(".")
    def on_test_error(self):
        self._formatter.write("E")
    def on_test_failure(self):
        self._formatter.write("F")
    def on_test_skip(self):
        self._formatter.write("S")
    def on_result_summary(self):
        self._formatter.writeln("")



import logbook
from confetti import Config
from .utils.conf_utils import Doc, Cmdline

__all__ = ["config"]

config = Config({
    "debug" : {
        "debug_skips" : False // Doc("Enter pdb also for SkipTest exceptions"),
        "enabled" : False // Doc("Enter pdb on failures and errors") // Cmdline(on="--pdb"),
    },
    "log" : {
        "console_level" : logbook.WARNING // Doc("console verbosity") // Cmdline(decrease="-v", increase="-q"),
        "root" : None // Doc("Root directory for logs") // Cmdline(arg="-l", metavar="DIR"),
        "subpath" : "{context.session.id}/{context.test_id}/log" // Doc("Path to write logs to under the root"),
        "session_subpath" : "session.log",
    },
    "run" : {
        "stop_on_error" : False // Doc("Stop execution when a test doesn't succeed") // Cmdline(on="-x"),
    },
    "notifications" : {
        "prowl_api_key" : None,
        "nma_api_key" : None,
    },
    "sentry" : {
        "dsn" : None // Doc("Possible DSN for a sentry service to log swallowed exceptions. "
                            "See http://getsentry.com for details"),
    },
    "hooks" : {
        "swallow_exceptions" : False // Doc("If set, exceptions inside hooks will be re-raised"),
    },
    "plugins" : {
        "search_paths" : [] // Doc("List of paths in which to search for plugin modules"),
    },
})



# -*- coding: utf-8 -*-
"""
slash.local - context-local objects.

The source code in this file is derived from werkzeug's local.py (Copyright 2011 by the Werkzeug Team)

The license for the original code follows:

8< - - - - - - - - - - 8< - - - - - - - - - - 8< - - - - - - - - - - 8< - - - - - - - - - -
Copyright (c) 2011 by the Werkzeug Team, see AUTHORS for more details.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.

    * Redistributions in binary form must reproduce the above
      copyright notice, this list of conditions and the following
      disclaimer in the documentation and/or other materials provided
      with the distribution.

    * The names of the contributors may not be used to endorse or
      promote products derived from this software without specific
      prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
"AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
"""
# since each thread has its own greenlet we can just use those as identifiers
# for the context.  If greenlets are not available we fall back to the
# current thread ident.
import six # pylint: disable=F0401
try:
    from greenlet import getcurrent as get_ident # pylint: disable=F0401
except ImportError: # pragma: no cover
    try:
        if six.PY3:
            try:
                from threading import get_ident # pylint: disable=F0401,E0611
            except ImportError:
                from threading import _get_ident as get_ident # for Python3.2 specifically
        else:
            from thread import get_ident # pylint: disable=F0401
    except ImportError: # pragma: no cover
        from dummy_thread import get_ident # pylint: disable=F0401

def release_local(local):
    """Releases the contents of the local for the current context.
    This makes it possible to use locals without a manager.

    Example::

        >>> loc = Local()
        >>> loc.foo = 42
        >>> release_local(loc)
        >>> hasattr(loc, 'foo')
        False

    With this function one can release :class:`Local` objects as well
    as :class:`StackLocal` objects.  However it is not possible to
    release data held by proxies that way, one always has to retain
    a reference to the underlying local object in order to be able
    to release it.

    .. versionadded:: 0.6.1
    """
    local.__release_local__()


class Local(object):
    __slots__ = ('__storage__', '__ident_func__')

    def __init__(self):
        object.__setattr__(self, '__storage__', {})
        object.__setattr__(self, '__ident_func__', get_ident)

    def __iter__(self):
        return iter(self.__storage__.items())

    def __call__(self, proxy):
        """Create a proxy for a name."""
        return LocalProxy(self, proxy)

    def __release_local__(self):
        self.__storage__.pop(self.__ident_func__(), None)

    def __getattr__(self, name):
        try:
            return self.__storage__[self.__ident_func__()][name]
        except KeyError:
            raise AttributeError(name)

    def __setattr__(self, name, value):
        ident = self.__ident_func__()
        storage = self.__storage__
        try:
            storage[ident][name] = value
        except KeyError:
            storage[ident] = {name: value}

    def __delattr__(self, name):
        try:
            del self.__storage__[self.__ident_func__()][name]
        except KeyError:
            raise AttributeError(name)


class LocalStack(object):
    """This class works similar to a :class:`Local` but keeps a stack
    of objects instead.  This is best explained with an example::

        >>> ls = LocalStack()
        >>> ls.push(42)
        >>> ls.top
        42
        >>> ls.push(23)
        >>> ls.top
        23
        >>> ls.pop()
        23
        >>> ls.top
        42

    They can be force released by using a :class:`LocalManager` or with
    the :func:`release_local` function but the correct way is to pop the
    item from the stack after using.  When the stack is empty it will
    no longer be bound to the current context (and as such released).

    By calling the stack without arguments it returns a proxy that resolves to
    the topmost item on the stack.

    .. versionadded:: 0.6.1
    """

    def __init__(self):
        self._local = Local()

    def __release_local__(self):
        self._local.__release_local__()

    def _get__ident_func__(self):
        return self._local.__ident_func__
    def _set__ident_func__(self, value):
        object.__setattr__(self._local, '__ident_func__', value)
    __ident_func__ = property(_get__ident_func__, _set__ident_func__)
    del _get__ident_func__, _set__ident_func__

    def __call__(self):
        def _lookup():
            rv = self.top
            if rv is None:
                raise RuntimeError('object unbound')
            return rv
        return LocalProxy(_lookup)

    def push(self, obj):
        """Pushes a new item to the stack"""
        rv = getattr(self._local, 'stack', None)
        if rv is None:
            self._local.stack = rv = [] # pylint: disable=W0201
        rv.append(obj)
        return rv

    def pop(self):
        """Removes the topmost item from the stack, will return the
        old value or `None` if the stack was already empty.
        """
        stack = getattr(self._local, 'stack', None)
        if stack is None:
            return None
        elif len(stack) == 1:
            release_local(self._local)
            return stack[-1]
        else:
            return stack.pop()

    @property
    def top(self):
        """The topmost item on the stack.  If the stack is empty,
        `None` is returned.
        """
        try:
            return self._local.stack[-1]
        except (AttributeError, IndexError):
            return None


class LocalManager(object):
    """Local objects cannot manage themselves. For that you need a local
    manager.  You can pass a local manager multiple locals or add them later
    by appending them to `manager.locals`.  Everytime the manager cleans up
    it, will clean up all the data left in the locals for this context.

    The `ident_func` parameter can be added to override the default ident
    function for the wrapped locals.

    .. versionchanged:: 0.6.1
       Instead of a manager the :func:`release_local` function can be used
       as well.

    .. versionchanged:: 0.7
       `ident_func` was added.
    """

    def __init__(self, locals=None, ident_func=None): # pylint: disable=W0622
        if locals is None:
            self.locals = []
        elif isinstance(locals, Local):
            self.locals = [locals]
        else:
            self.locals = list(locals)
        if ident_func is not None:
            self.ident_func = ident_func
            for local in self.locals:
                object.__setattr__(local, '__ident_func__', ident_func)
        else:
            self.ident_func = get_ident

    def get_ident(self):
        """Return the context identifier the local objects use internally for
        this context.  You cannot override this method to change the behavior
        but use it to link other context local objects (such as SQLAlchemy's
        scoped sessions) to the Werkzeug locals.

        .. versionchanged:: 0.7
           Yu can pass a different ident function to the local manager that
           will then be propagated to all the locals passed to the
           constructor.
        """
        return self.ident_func()

    def cleanup(self):
        """Manually clean up the data in the locals for this context.  Call
        this at the end of the request or use `make_middleware()`.
        """
        for local in self.locals:
            release_local(local)

    def __repr__(self):
        return '<%s storages: %d>' % (
            self.__class__.__name__,
            len(self.locals)
        )


class LocalProxy(object):
    """Acts as a proxy for a werkzeug local.  Forwards all operations to
    a proxied object.  The only operations not supported for forwarding
    are right handed operands and any kind of assignment.

    Example usage::

        from werkzeug.local import Local
        l = Local()

        # these are proxies
        request = l('request')
        user = l('user')


        from werkzeug.local import LocalStack
        _response_local = LocalStack()

        # this is a proxy
        response = _response_local()

    Whenever something is bound to l.user / l.request the proxy objects
    will forward all operations.  If no object is bound a :exc:`RuntimeError`
    will be raised.

    To create proxies to :class:`Local` or :class:`LocalStack` objects,
    call the object as shown above.  If you want to have a proxy to an
    object looked up by a function, you can (as of Werkzeug 0.6.1) pass
    a function to the :class:`LocalProxy` constructor::

        session = LocalProxy(lambda: get_current_request().session)

    .. versionchanged:: 0.6.1
       The class can be instanciated with a callable as well now.
    """
    __slots__ = ('__local', '__dict__', '__name__')

    def __init__(self, local, name=None):
        object.__setattr__(self, '_LocalProxy__local', local)
        object.__setattr__(self, '__name__', name)

    def _get_current_object(self):
        """Return the current object.  This is useful if you want the real
        object behind the proxy at a time for performance reasons or because
        you want to pass the object into a different context.
        """
        if not hasattr(self.__local, '__release_local__'):
            return self.__local()
        try:
            return getattr(self.__local, self.__name__)
        except AttributeError:
            raise RuntimeError('no object bound to %s' % self.__name__)

    @property
    def __dict__(self):
        try:
            return self._get_current_object().__dict__
        except RuntimeError:
            raise AttributeError('__dict__')

    def __repr__(self):
        try:
            obj = self._get_current_object()
        except RuntimeError:
            return '<%s unbound>' % self.__class__.__name__
        return repr(obj)

    def __nonzero__(self):
        try:
            return bool(self._get_current_object())
        except RuntimeError:
            return False

    def __unicode__(self):
        try:
            return unicode(self._get_current_object()) # pylint: disable=E0602
        except RuntimeError:
            return repr(self)

    def __dir__(self):
        try:
            return dir(self._get_current_object())
        except RuntimeError:
            return []

    def __getattr__(self, name):
        if name == '__members__':
            return dir(self._get_current_object())
        return getattr(self._get_current_object(), name)

    def __setitem__(self, key, value):
        self._get_current_object()[key] = value

    def __delitem__(self, key):
        del self._get_current_object()[key]

    def __setslice__(self, i, j, seq):
        self._get_current_object()[i:j] = seq

    def __delslice__(self, i, j):
        del self._get_current_object()[i:j]

    __setattr__ = lambda x, n, v: setattr(x._get_current_object(), n, v) # pylint: disable=W0212
    __delattr__ = lambda x, n: delattr(x._get_current_object(), n) # pylint: disable=W0212
    __str__ = lambda x: str(x._get_current_object()) # pylint: disable=W0212
    __lt__ = lambda x, o: x._get_current_object() < o # pylint: disable=W0212
    __le__ = lambda x, o: x._get_current_object() <= o # pylint: disable=W0212
    __eq__ = lambda x, o: x._get_current_object() == o # pylint: disable=W0212
    __ne__ = lambda x, o: x._get_current_object() != o # pylint: disable=W0212
    __gt__ = lambda x, o: x._get_current_object() > o # pylint: disable=W0212
    __ge__ = lambda x, o: x._get_current_object() >= o # pylint: disable=W0212
    __cmp__ = lambda x, o: cmp(x._get_current_object(), o) # pylint: disable=W0212,E0602
    __hash__ = lambda x: hash(x._get_current_object()) # pylint: disable=W0212
    __call__ = lambda x, *a, **kw: x._get_current_object()(*a, **kw) # pylint: disable=W0212,W0142
    __len__ = lambda x: len(x._get_current_object()) # pylint: disable=W0212
    __getitem__ = lambda x, i: x._get_current_object()[i] # pylint: disable=W0212
    __iter__ = lambda x: iter(x._get_current_object()) # pylint: disable=W0212
    __contains__ = lambda x, i: i in x._get_current_object() # pylint: disable=W0212
    __getslice__ = lambda x, i, j: x._get_current_object()[i:j] # pylint: disable=W0212
    __add__ = lambda x, o: x._get_current_object() + o # pylint: disable=W0212
    __sub__ = lambda x, o: x._get_current_object() - o # pylint: disable=W0212
    __mul__ = lambda x, o: x._get_current_object() * o # pylint: disable=W0212
    __floordiv__ = lambda x, o: x._get_current_object() // o # pylint: disable=W0212
    __mod__ = lambda x, o: x._get_current_object() % o # pylint: disable=W0212
    __divmod__ = lambda x, o: x._get_current_object().__divmod__(o) # pylint: disable=W0212
    __pow__ = lambda x, o: x._get_current_object() ** o # pylint: disable=W0212
    __lshift__ = lambda x, o: x._get_current_object() << o # pylint: disable=W0212
    __rshift__ = lambda x, o: x._get_current_object() >> o # pylint: disable=W0212
    __and__ = lambda x, o: x._get_current_object() & o # pylint: disable=W0212
    __xor__ = lambda x, o: x._get_current_object() ^ o # pylint: disable=W0212
    __or__ = lambda x, o: x._get_current_object() | o # pylint: disable=W0212
    __div__ = lambda x, o: x._get_current_object().__div__(o) # pylint: disable=W0212
    __truediv__ = lambda x, o: x._get_current_object().__truediv__(o) # pylint: disable=W0212
    __neg__ = lambda x: -(x._get_current_object()) # pylint: disable=W0212
    __pos__ = lambda x: +(x._get_current_object()) # pylint: disable=W0212
    __abs__ = lambda x: abs(x._get_current_object()) # pylint: disable=W0212
    __invert__ = lambda x: ~(x._get_current_object()) # pylint: disable=W0212
    __complex__ = lambda x: complex(x._get_current_object()) # pylint: disable=W0212
    __int__ = lambda x: int(x._get_current_object()) # pylint: disable=W0212
    __long__ = lambda x: long(x._get_current_object()) # pylint: disable=W0212,E0602
    __float__ = lambda x: float(x._get_current_object()) # pylint: disable=W0212
    __oct__ = lambda x: oct(x._get_current_object()) # pylint: disable=W0212
    __hex__ = lambda x: hex(x._get_current_object()) # pylint: disable=W0212
    __index__ = lambda x: x._get_current_object().__index__() # pylint: disable=W0212
    __coerce__ = lambda x, o: x._get_current_object().__coerce__(x, o) # pylint: disable=W0212,W0108
    __enter__ = lambda x: x._get_current_object().__enter__() # pylint: disable=W0212
    __exit__ = lambda x, *a, **kw: x._get_current_object().__exit__(*a, **kw) # pylint: disable=W0212,W0142



from . import context
from .conf import config
from .utils.path import ensure_containing_directory
from contextlib import contextmanager
import logbook # pylint: disable=F0401
from logbook.more import ColorizedStderrHandler # pylint: disable=F0401
import os

@contextmanager
def get_test_logging_context():
    handler = _get_file_log_handler(config.root.log.subpath)
    with handler:
        with _get_console_handler():
            yield

@contextmanager
def get_session_logging_context():
    handler = _get_file_log_handler(config.root.log.session_subpath)
    with handler:
        with _get_console_handler():
            yield

def _get_console_handler():
    return ColorizedStderrHandler(bubble=True, level=config.root.log.console_level)

def _get_file_log_handler(subpath):
    root_path = config.root.log.root
    if root_path is None:
        handler = logbook.NullHandler(bubble=False)
    else:
        log_path = os.path.join(root_path, subpath.format(context=context))
        ensure_containing_directory(log_path)
        handler = logbook.FileHandler(log_path, bubble=False)
    return handler




class SlashException(Exception):
    pass

class CannotLoadTests(SlashException):
    pass

class TestFailed(AssertionError):
    """
    This exception class distinguishes actual test failures (mostly assertion errors,
    but possibly other conditions as well) from regular asserts.

    This is important, since regular code that is tested can use asserts, and that
    should not be considered a test failure (but rather a code failure)
    """
    pass

class SkipTest(Exception):
    """
    This exception should be raised in order to interrupt the execution of the currently running test, marking
    it as skipped
    """
    def __init__(self, reason="Test skipped"):
        super(SkipTest, self).__init__(reason)
        self.reason = reason



_NOTHING = object()
_END = object()

class PeekableIterator(object):
    """An iterator wrapper which allows peeking into the next element"""
    def __init__(self, iterator):
        super(PeekableIterator, self).__init__()
        self._iterator = iter(iterator)
        self._peeked = _NOTHING

    def __iter__(self):
        return self

    def next(self):
        if self._peeked is not _NOTHING:
            returned = self._peeked
            self._peeked = _NOTHING
            if returned is _END:
                raise StopIteration()
            return returned
        return next(self._iterator)
    __next__ = next

    def peek_or_none(self):
        if self.has_next():
            return self.peek()
        return None

    def peek(self):
        if self._peeked is _NOTHING:
            try:
                self._peeked = next(self._iterator)
            except StopIteration:
                self._peeked = _END
        if self._peeked is _END:
            raise StopIteration()
        return self._peeked

    def has_next(self):
        try:
            self.peek()
        except StopIteration:
            return False
        return True



"""
Slash python compatibility library for cross-python support
"""
#pylint: disable=W0611
import sys

PY2 = (sys.version_info[0] == 2)

if PY2:
    from __builtin__ import xrange
else:
    xrange = range



from .utils import TestCase
from .utils.test_generator import TestGenerator
from slash.loader import Loader
from slash.session import Session

from uuid import uuid1
import shutil
import sys
import os

class TestRepositoryTest(TestCase):
    def setUp(self):
        super(TestRepositoryTest, self).setUp()
        self.generator = TestGenerator()
        self.package_name = "package_{0}".format(str(uuid1()).replace("-", "_"))
        self.root = self.generator.write_test_directory({
            self.package_name: {
                "__init__.py": "",
                "tests" : {
                    "__init__.py": "",
                    "dir1": {
                        "__init__.py": "",
                        "dir2": {
                            "__init__.py": "",
                            "test_1.py": self.generator.generate_test(),
                            "test_2.py": self.generator.generate_test(),
                        },
                    }}}})
        self.addCleanup(shutil.rmtree, self.root)

class ImportErrorsTest(TestRepositoryTest):
    def setUp(self):
        super(ImportErrorsTest, self).setUp()
        self.path = os.path.join(self.root, self.package_name, "tests", "dir1", "dir2")
        with open(os.path.join(self.path, "test_3.py"), "w") as f:
            f.write(":::")
        with open(os.path.join(self.path, "test_4.py"), "w") as f:
            f.write("import nonexistent_module_here")

    def test_import_errors_without_session(self):
        with self.assertRaises((SyntaxError, ImportError)):
            list(Loader().iter_path(self.root))

    def test_import_errors_with_session(self):
        with Session() as s:
            tests = list(Loader().iter_path(self.root))
        self.assertTrue(tests)
        self.assertFalse(s.result.global_result.is_success())
        errors = s.result.global_result.get_errors()
        self._assert_file_failed_with(errors, "test_3.py", SyntaxError)
        self._assert_file_failed_with(errors, "test_4.py", ImportError)

    def _assert_file_failed_with(self, errors, filename, error_type):
        [err] = [e for e in errors if e.exception_type is error_type]
        self.assertIn(filename, err.exception_text)

class PathLoadingTest(TestRepositoryTest):
    def test_iter_path(self):
        self.assert_all_discovered(Loader().iter_path(self.root))
    def test_iter_package_import_first(self):
        self._test_iter_package(True, False)
    def test_iter_package_import_first_force_init_py(self):
        self._test_iter_package(True, True)
    def test_iter_package_dont_import_first(self):
        self._test_iter_package(False, False)
    def _test_iter_package(self, import_first, force_init_py):
        sys.path.insert(0, self.root)
        package_name = "{0}.tests".format(self.package_name)
        if import_first:
            __import__(self.package_name)
            filename = sys.modules[self.package_name].__file__
            if force_init_py and not filename.endswith("__init__.py") and not filename.endswith("__init__.pyc"):
                sys.modules[self.package_name].__file__ = os.path.join(filename, "__init__.py")
        self.assert_all_discovered(Loader().iter_package(self.package_name))
        self.assertIn(self.package_name, sys.modules)
    def assert_all_discovered(self, iterator):
        tests = list(iterator)
        self.assertEquals(
            set(test.TESTGENERATOR_TEST_ID for test in tests),
            set(self.generator.get_expected_test_ids())
            )



import itertools
import sys
from .utils.error_object import Error

class Result(object):
    def __init__(self, test_metadata=None):
        super(Result, self).__init__()
        self.test_metadata = test_metadata
        self._errors = []
        self._failures = []
        self._skips = []
        self._finished = False
        self._interrupted = False
    def is_error(self):
        return bool(self._errors)
    def is_failure(self):
        return bool(self._failures)
    def is_just_failure(self):
        """Indicates this is a pure failure, without errors involved"""
        return self.is_failure() and not self.is_error()
    def is_skip(self):
        return bool(self._skips)
    def is_success(self):
        return not self._errors and not self._failures and not self._skips and not self._interrupted
    def is_success_finished(self):
        return self.is_success() and self.is_finished()
    def is_finished(self):
        return self._finished
    def mark_finished(self):
        self._finished = True
    def mark_interrupted(self):
        self._interrupted = True
    def is_interrupted(self):
        return self._interrupted
    def add_error(self):
        self._errors.append(Error(sys.exc_info()))
    def add_failure(self):
        self._failures.append(Error(sys.exc_info()))
    def add_skip(self, reason):
        self._skips.append(reason)
    def get_errors(self):
        return self._errors
    def get_failures(self):
        return self._failures
    def get_skips(self):
        return self._skips
    def __repr__(self):
        return "< Result ({0})>".format(
            ", ".join(
                attr
                for attr in ("success", "error", "failure", "skip", "finished", "interrupted")
                if getattr(self, "is_{0}".format(attr))()
                )
            )

class GlobalResult(Result):
    pass

class SessionResult(object):
    def __init__(self, test_result_iterator_func):
        super(SessionResult, self).__init__()
        self.global_result = GlobalResult()
        self._iterator = test_result_iterator_func
    def __iter__(self):
        return self._iterator()
    def is_success(self):
        return self.global_result.is_success() and all(result.is_finished() and result.is_success() for result in self._iterator())
    def get_num_successful(self):
        return self._count(Result.is_success_finished, include_global=False)
    def get_num_errors(self):
        return self._count(Result.is_error)
    def get_num_failures(self):
        return self._count(Result.is_just_failure)
    def get_num_skipped(self):
        return self._count(Result.is_skip)
    def _count(self, pred, include_global=True):
        returned = 0
        iterator = self.iter_all_results() if include_global else self.iter_test_results()
        for result in iterator:
            if pred(result):
                returned += 1
        return returned
    def iter_test_results(self):
        return iter(self)
    def iter_all_results(self):
        return itertools.chain(self.iter_test_results(), [self.global_result])



from .local import LocalStack
from .local import LocalProxy
import functools

__all__ = ["context", "session", "test", "test_id", "g", "internal_globals"]

class GlobalStorage(object):
    pass

class Context(object):
    session = test = test_id = None
    def __init__(self):
        super(Context, self).__init__()
        self.g = GlobalStorage()
        self.internal_globals = GlobalStorage()

class NullContext(object):
    def __setattr__(self, attr, value):
        raise AttributeError("Cannot set attribute {0!r} on null context".format(attr))
    @property
    def _always_none(self):
        pass
    session = test = test_id = g = internal_globals = _always_none

_ctx = LocalStack()
_ctx.push(NullContext())
context = _ctx() # proxy

def _lookup_object(name):
    top = _ctx.top
    if top is None:
        raise RuntimeError('Context stack is empty')
    return getattr(top, name)


session = LocalProxy(functools.partial(_lookup_object, "session"))
test    = LocalProxy(functools.partial(_lookup_object, "test"))
test_id    = LocalProxy(functools.partial(_lookup_object, "test_id"))
g = LocalProxy(functools.partial(_lookup_object, "g"))
internal_globals = LocalProxy(functools.partial(_lookup_object, "internal_globals"))

def push_context():
    _ctx.push(Context())
def pop_context():
    if isinstance(_ctx.top, NullContext):
        raise RuntimeError("Attempt to pop root context")
    _ctx.pop()



#! /usr/bin/python
from __future__ import print_function
import platform
import subprocess
import sys
import os

def _execute(cmd):
    if 0 != subprocess.call(cmd, shell=True):
        sys.exit(-1)

if __name__ == '__main__':
    print("Running from", os.path.abspath("."))
    if platform.python_version() < "3.3":
        _execute("pylint --rcfile=.pylintrc setup.py")
        _execute("pylint --rcfile=.pylintrc slash")
    _execute("nosetests -w tests")



import collections
from contextlib import contextmanager
import functools
import logbook

from .cleanups import add_cleanup
from .compat import xrange
from .ctx import internal_globals

_logger = logbook.Logger(__name__)

class TestContext(object):
    """
    Represents a context in which test cases are run
    """

    def before(self):
        """Called when the context kicks into effect"""
        pass

    def after(self):
        """Called when the context is discarded when no longer needed"""
        pass

    def before_case(self):
        """Called before each test case is run in this context"""
        pass

    def after_case(self):
        """
        Called after each test case is run in this context
        (regardless of success/failure status)
        """
        pass

def _add_context_to_class(context, cls):
    if cls.__slash_needed_contexts__ is None:
        cls.__slash_needed_contexts__ = collections.deque()
    # we call appendleft because class decorators are executed bottom-up
    # this will cause iteration to happen in the right order
    cls.__slash_needed_contexts__.appendleft(context)
    return cls

def with_context(ctx):
    return functools.partial(_add_context_to_class, ctx)

@contextmanager
def get_test_context_setup(current_test, next_test):
    """
    Given a set of test contexts, sets up all uninitialized test contexts before entry.

    After the context is departed, each context not needed anymore is torn down.
    """
    needed_contexts = _get_needed_contexts(current_test)
    next_needed_contexts = _get_needed_contexts(next_test)
    currently_active = _get_currently_active_contexts()
    for needed_context in needed_contexts:
        currently_active.push(needed_context)
    currently_active.trigger_before_case()
    add_cleanup(currently_active.trigger_after_case)
    add_cleanup(currently_active.pop_all_except, next_needed_contexts)
    yield

def _get_needed_contexts(test):
    needed = getattr(getattr(getattr(test, "__slash__", None), "factory", None), "__slash_needed_contexts__", None)
    if needed is None:
        needed = ()
    return needed

def _get_currently_active_contexts():
    returned = getattr(internal_globals, "active_test_contexts", None)
    if returned is None:
        returned = internal_globals.active_test_contexts = ActiveContexts()
    return returned

class ActiveContexts(object):
    def __init__(self):
        super(ActiveContexts, self).__init__()
        self.stack = []
        self.by_type = {}
    def push(self, context_type):
        if context_type in self.by_type:
            return
        ctx = context_type()
        self.by_type[context_type] = ctx
        self.stack.append(ctx)
        ctx.before()

    def pop_all_except(self, needed_context_types):
        needed_context_types = set(needed_context_types)
        for index in xrange(len(self.stack)-1, -1, -1):
            ctx = self.stack[index]
            if type(ctx) not in needed_context_types:
                _logger.debug("{0} no longer needed. Discarding...", ctx)
                self.stack.pop(index)
                self.by_type.pop(type(ctx))
                ctx.after()

    def trigger_before_case(self):
        for ctx in self.stack:
            ctx.before_case()
    def trigger_after_case(self):
        for ctx in reversed(self.stack):
            ctx.after_case()



from .utils import TestCase
from .utils import run_tests_assert_success
from tempfile import mkdtemp
import functools
import os
import slash

_IDENTIFIER = "logging-test"
_SESSION_START_MARK = "session-start-mark"
_SESSION_END_MARK = "session-end-mark"

class LoggingTest(TestCase):
    def test(self):
        self.log_path = mkdtemp()
        self.override_config(
            "log.root",
            self.log_path,
        )
        self.override_config(
            "log.subpath",
            os.path.join("{context.session.id}", "{context.test_id}", "debug.log")
        )
        self.override_config(
            "log.session_subpath",
            os.path.join("{context.session.id}", "debug.log")
        )

        slash.hooks.session_start.register(functools.partial(_mark, _SESSION_START_MARK), identifier=_IDENTIFIER)
        self.addCleanup(slash.hooks.session_start.unregister_by_identifier, _IDENTIFIER)

        slash.hooks.session_end.register(functools.partial(_mark, _SESSION_END_MARK), identifier=_IDENTIFIER)
        self.addCleanup(slash.hooks.session_end.unregister_by_identifier, _IDENTIFIER)

        self.session = run_tests_assert_success(SampleTest)
        self.test_ids = [result.test_metadata.id for result in self.session.result.iter_test_results()]
        self._test_all_run()
        self._test_test_logs_written()
        self._test_session_logs()

    def _test_all_run(self):
        methods = [
            method_name for method_name in dir(SampleTest)
            if method_name.startswith("test")
            ]
        self.assertTrue(methods)
        self.assertEquals(len(self.test_ids), len(methods))

    def _test_test_logs_written(self):
        for test_id in self.test_ids:
            log_path = os.path.join(self.log_path, self.session.id, test_id, "debug.log")
            with open(log_path) as f:
                data = f.read()
            for other_test_id in self.test_ids:
                if other_test_id != test_id:
                    self.assertNotIn(other_test_id, data)
            self.assertNotIn(_SESSION_START_MARK, data)
            self.assertNotIn(_SESSION_END_MARK, data)

    def _test_session_logs(self):
        with open(os.path.join(self.log_path, self.session.id, "debug.log")) as f:
            data = f.read()
        self.assertIn(_SESSION_START_MARK, data)
        self.assertIn(_SESSION_END_MARK, data)
        for test_id in self.test_ids:
            self.assertNotIn(test_id, data)

class SampleTest(slash.Test):
    def test_1(self):
        _mark()
    def test_2(self):
        _mark()

def _mark(text=None):
    if text is None:
        text = slash.context.test_id
    slash.logger.debug(text)



from .utils import TestCase
from slash.result import Result
from slash.result import SessionResult

class SessionResultTest(TestCase):
    def setUp(self):
        super(SessionResultTest, self).setUp()
        results = [
            Result() for _ in range(10)
            ]
        # one result with both errors and failures
        try:
            1 / 0
        except:
            results[1].add_error()
            results[1].add_failure()
            # one result with failure
            results[2].add_failure()
            # one result with error
            results[3].add_error()
            results[5].add_error()

        # one result will skip
        results[4].add_skip("Reason")

        # and one result will skip with error
        results[5].add_skip("Reason")
        num_finished = 7

        for result in results[:num_finished]:
            result.mark_finished()
        self.result = SessionResult(results.__iter__)

    def test_counts(self):
        self.assertEquals(self.result.get_num_successful(), 2)
        # errors take precedence over failures
        self.assertEquals(self.result.get_num_errors(), 3)
        self.assertEquals(self.result.get_num_skipped(), 2)
        self.assertEquals(self.result.get_num_failures(), 1)



class Metadata(object):
    def __init__(self, test):
        super(Metadata, self).__init__()
        self.id = None
        self.factory = None
        # we don't keep a copy of the test itself. The metadata should be preserved
        # separately, saving memory for very large runs
        self.canonical_name = test.get_canonical_name()
    def __repr__(self):
        return self.canonical_name

def ensure_slash_metadata(test):
    returned = getattr(test, "__slash__", None)
    if returned is None:
        returned = test.__slash__ = Metadata(test)
    return returned



from collections import deque
from .ctx import context
import logbook

_logger = logbook.Logger(__name__)

class _Cleanup(object):
    def __init__(self, func, args, kwargs, critical=False):
        super(_Cleanup, self).__init__()
        self.func = func
        self.args = args
        self.kwargs = kwargs
        self.critical = critical
    def __call__(self):
        return self.func(*self.args, **self.kwargs) # pylint: disable=W0142

def add_cleanup(_func, *args, **kwargs):
    """
    Adds a cleanup function to the cleanup stack. Cleanups are executed in a LIFO order.

    Positional arguments and keywords are passed to the cleanup function when called.
    """
    _add_cleanup(_Cleanup(_func, args, kwargs))

def add_critical_cleanup(_func, *args, **kwargs):
    """
    Same as :func:`.add_cleanup`, only the cleanup will be called even on interrupted tests
    """
    _add_cleanup(_Cleanup(_func, args, kwargs, critical=True))

def _add_cleanup(cleanup):
    _get_cleanups().append(cleanup)

def call_cleanups(critical_only=False):
    cleanups = _get_cleanups()
    while cleanups:
        cleanup = cleanups.popleft()
        if critical_only and not cleanup.critical:
            continue
        try:
            cleanup()
        except:
            context.session.get_result(context.test).add_error()

def _get_cleanups():
    returned = getattr(context, "cleanups", None)
    if returned is None:
        returned = context.cleanups = deque()
    return returned



from .utils import TestCase
import slash

class GlobalStorageTest(TestCase):
    hook_called = False
    identifier = object()
    def test_global_storage_exists_on_session_start(self):
        @slash.exception_handling.disable_exception_swallowing
        def _on_session_start():
            self.assertIsNotNone(slash.g)
            slash.g.value = "value"
            self.assertEquals(slash.g.value, "value")
            self.hook_called = True
        slash.hooks.session_start.register(_on_session_start, self.identifier)
        self.addCleanup(
            slash.hooks.session_start.unregister_by_identifier,
            self.identifier
        )
        with slash.session.Session():
            pass
        self.assertTrue(self.hook_called)



import functools
from .utils import skip_test
from .parameters import iterate_kwargs_options
from .runnable_test import RunnableTest
from .runnable_test_factory import RunnableTestFactory

class Test(RunnableTest, RunnableTestFactory):
    """
    This is a base class for implementing unittest-style test classes.
    """
    def __init__(self, test_method_name, before_kwargs=None, after_kwargs=None, test_kwargs=None):
        super(Test, self).__init__()
        self._test_method_name = test_method_name
        self._before_kwargs = before_kwargs or {}
        self._after_kwargs = after_kwargs or {}
        self._test_kwargs = test_kwargs or {}
    __slash_skipped__ = False
    __slash_skipped_reason__ = None
    @classmethod
    def skip_all(cls, reason=None):
        cls.__slash_skipped__ = True
        cls.__slash_skipped_reason__ = reason

    @classmethod
    def _generate_tests(cls):
        if is_abstract_base_class(cls):
            return

        before_kwarg_sets = list(iterate_kwargs_options(cls.before))
        after_kwarg_sets = list(iterate_kwargs_options(cls.after))
        for test_method_name in dir(cls):
            if not test_method_name.startswith("test"):
                continue
            test_method = getattr(cls, test_method_name)
            for before_kwargs in before_kwarg_sets:
                for test_kwargs in iterate_kwargs_options(test_method):
                    for after_kwargs in after_kwarg_sets:
                        case = cls(
                            test_method_name,
                            before_kwargs=before_kwargs,
                            test_kwargs=test_kwargs,
                            after_kwargs=after_kwargs
                        )
                        if cls.__slash_skipped__:
                            case.run = functools.partial(
                                skip_test,
                                cls.__slash_skipped_reason__
                            )
                        yield case
    def run(self): # pylint: disable=E0202
        """
        Not to be overriden
        """
        method = getattr(self, self._test_method_name)
        self.before(**self._before_kwargs)
        try:
            method(**self._test_kwargs)
        finally:
            self.after(**self._after_kwargs)
    def before(self):
        """
        Gets called before each separate case generated from this test class
        """
        pass
    def after(self):
        """
        Gets called after each separate case from this test class executed, assuming :meth:`before` was successful.
        """
        pass
    def get_canonical_name(self):
        return "{0}:{1}".format(super(Test, self).get_canonical_name(), self._test_method_name)


def abstract_test_class(cls):
    """
    Marks a class as **abstract**, thus meaning it is not to be run
    directly, but rather via a subclass.
    """
    assert issubclass(cls, Test), "abstract_test_class only operates on slash.Test subclasses"
    cls.__slash_abstract__ = True
    return cls

def is_abstract_base_class(cls):
    """
    Checks if a given class is abstract.

    .. seealso:: :func:`abstract_test_class`
    """
    return bool(cls.__dict__.get("__slash_abstract__", False))



from .. import hooks

class HooksContextManager(object):
    """
    This class can be used to register and unregister multiple hooks
    within a context. The callbacks are collected by searching methods
    prefixed by "on_<hook name>" (.e.g: on_test_error).
    """
    PREFIX = "on_"
    def __init__(self):
        self._id = object()
    def _get_hooks_and_callbacks(self):
        return [(getattr(hooks, member.replace(self.PREFIX, "")), # the hook
                 getattr(self, member)) # the callback
                for member in dir(self)
                if member.startswith(self.PREFIX)]
    def __enter__(self):
        for hook, callback in self._get_hooks_and_callbacks():
            hook.register(callback, self._id)
        return self
    def __exit__(self, *e):
        for hook, _ in self._get_hooks_and_callbacks():
            hook.unregister_by_identifier(self._id)


import slash
from slash.exceptions import (
    SkipTest,
    TestFailed,
    )
from tempfile import mkdtemp
import itertools
import os
import six # pylint: disable=F0401
import uuid
import logbook

_logger = logbook.Logger(__name__)

_current_test_generator = None

class TestGenerator(object):
    """
    A utility for generating test classes and making sure they actually
    run
    """
    __test__ = False # for nose

    def __init__(self):
        super(TestGenerator, self).__init__()
        self._expected = set()
        self._uuid_base = str(uuid.uuid1())
        self._uuid_generator = ("{0}/{1}".format(self._uuid_base, i) for i in itertools.count(1))
        self._run_callbacks = {}
        global _current_test_generator
        _current_test_generator = self
    def get_expected_test_ids(self):
        return list(self._expected)
    def generate_tests(self, num_tests=3):
        return [self.generate_test() for _ in six.moves.xrange(num_tests)]
    def generate_test(self):
        test_id = next(self._uuid_generator)
        self._expected.add(test_id)
        return TestPromise(test_id)
    def write_test_directory(self, structure, root=None):
        if root is None:
            root = mkdtemp()
        if not os.path.isdir(root):
            os.makedirs(root)
        for filename, item in six.iteritems(structure):
            path = os.path.join(root, filename)
            if isinstance(item, dict):
                self.write_test_directory(item, path)
            else:
                contents = self._get_test_item_contents(item)
                with open(path, "w") as f:
                    f.write(contents)
        return root

    def _get_test_item_contents(self, items):
        if isinstance(items, str):
            return items
        if not isinstance(items, list):
            items = [items]
        return "\n".join(itertools.chain(
            [_TESTFILE_HEADER],
            (item.generate_test_factory_source() for item in items)
        ))

    def notify_run(self, test_id, test):
        _logger.debug("Running: {0}", test_id)
        self._expected.remove(test_id)
        callback_list = self._run_callbacks.get(test_id, [])
        while callback_list:
            callback_list.pop(0)(test)
    def assert_all_run(self):
        assert not self._expected, "These tests have not run: {0}".format(", ".join(self._expected))

    ### Functions to manage test execution
    def make_test_raise_exception(self, testpromise):
        self.add_test_run_callback(testpromise, self.do_raise_exception)
    def make_test_fail(self, testpromise):
        self.add_test_run_callback(testpromise, self.do_fail)
    def make_test_skip(self, testpromise):
        self.add_test_run_callback(testpromise, self.do_skip)
    def add_test_run_callback(self, testpromise, handler):
        test_promise_id = self._get_test_promise_id(testpromise)
        _logger.debug("Adding test run callback: {0} ==> {1}", test_promise_id, handler)
        self._run_callbacks.setdefault(test_promise_id, []).append(handler)
    def _get_test_promise_id(self, p):
        if isinstance(p, slash.RunnableTest):
            p = p.__test_generator_promise__
        if isinstance(p, str):
            return p # test id
        return p.id
    ### handlers
    def do_raise_exception(self, _):
        raise OSError("Sample exception")
    def do_fail(self, _):
        raise TestFailed("Test failed")
    def do_skip(self, _):
        raise SkipTest("Reason here")

class TestPromise(object):
    __test__ = False # for nose
    def __init__(self, test_promise_id):
        super(TestPromise, self).__init__()
        self.id = test_promise_id
        self._test_class_name = "T" + self.id.replace("-", "_").replace("/", "_")
    def generate_test_factory_source(self):
        return _TEST_FACTORY_SOURCE_TEMPLATE.format(**self._get_template_context())
    def generate_test_source(self):
        return _TEST_SOURCE_TEMPLATE.format(**self._get_template_context())
    def _get_template_context(self):
        return {
            "test_generator_module_name" : __name__,
            "bucket_test_id" : self.id,
            "test_class_name" : self._test_class_name,
        }
    def generate_test(self):
        d = {"RunnableTest" : slash.RunnableTest}
        exec(self.generate_test_source(), d)
        returned = d[self._test_class_name]()
        returned.__test_generator_promise__ = self
        return returned

_TESTFILE_HEADER = """
from slash import RunnableTest, RunnableTestFactory
"""

_TEST_SOURCE_TEMPLATE = r"""
class {test_class_name}(RunnableTest):
    TESTGENERATOR_TEST_ID = {bucket_test_id!r}
    def run(self):
        from {test_generator_module_name} import _current_test_generator
        _current_test_generator.notify_run({bucket_test_id!r}, self)
"""

_TEST_FACTORY_SOURCE_TEMPLATE = """
class F{{test_class_name}}(RunnableTestFactory):
    @classmethod
    def generate_tests(cls):
{0}
        return [{{test_class_name}}()]
""".format("\n".join((" " * 8 + line) for line in _TEST_SOURCE_TEMPLATE.splitlines()))



import abc

class PluginInterface(object):
    """
    This class represents the base interface needed from plugin classes.
    """
    __metaclass__ = abc.ABCMeta

    def activate(self):
        """
        Called when the plugin is activated
        """
        pass

    def configure_argument_parser(self, parser):
        """
        Gives a chance to the plugin to add options received from comand-line
        """
        pass

    def configure_from_parsed_args(self, args):
        """
        Called after successful parsing of command-line arguments
        """
        pass

    def get_description(self):
        """
        Retrieves a quick description for this plugin, mostly used in command-line help or online documentation.
        It is not mandatory to override this method.
        """
        return None

    def get_name(self):
        """
        Returns the name of the plugin class. This name is used to register, disable and address
        the plugin during runtime.

        Note that the command-line switches (``--with-...``) are derived from this name.

        Any implemented plugin must override this method.
        """
        raise NotImplementedError() # pragma: no cover
